%%-------------------------------------------------
%% generate a finite difference matrix -- using fd3dB
%%-------------------------------------------------
addpath ../sparse;
addpath ../precon;
A = fd3d(40, 20, 5, 0.1, 0.2, 0.1, 0.2);      %% <--- 3-D dim n = 4000
%% [see eigenvalues at the end with eigs(A,20,'SR')]
n = size(A,1);
%%------------ params for fgmres
  im=100; maxits=200; tolIts=1.e-08;
spy(A) 
title([' Conv. Diffusion Type matrix -  n = ', num2str(n)]) 
disp('pausing -- push any key to continue ') 
  pause (0.3);
close all 
%%-------------------- create artificial rhs + initial guess
rhs = A * ([1:n]') ; 
sol0 = rand(n,1);
nz0 = nnz(A);
%%-------------------- set up SSOR preconditioner . with omega =1
 L = tril(A); D = diag(A); U = triu(A);
 SSOR.L = L; 
 SSOR.D = full(D);
 SSOR.U = U;
 SSOR.kiter = 10;
%%-------------------- call fgmres
 disp(' solution with SSOR prec') 
 tic
 [sol,res1,its] = fgmres(A, SSOR,'precLDU', rhs, sol0,im,maxits,tolIts) ;
 nzp = 0;
 sz1 = sprintf('%4.1f\n',nzp);
 toc
 disp(' ** done ** ') 
%%--------------------   ILU0 preconditioner  
 [L, U] = ilu(A) ;
 ILU = struct('L',L,'U',U) 
 [sol,res2,its] = fgmres(A, ILU,'precLU', rhs, sol0,im,maxits,tolIts) ;
 disp(' ** done ** ')
 nzp = 1;
 sz2 = sprintf('%4.1f\n',nzp);
%%--------------------- ILU(1) - [note: inefficient build for demo only]
 [L, U] = iluk(A,1) ;
 ILU.L = L; ILU.U = U;
 nzp = (nnz(L)+nnz(U)-n)/nz0;
 sz3 = sprintf('%4.1f\n',nzp);
 [sol,res3,its] = fgmres(A, ILU,'precLU', rhs, sol0,im,maxits,tolIts) ;
 disp(' ** done ** ') 
 %%-------------------- ILUT - crout variant
 disp(' solution with ILUT-GMRES')
 opts.type = 'crout';
 opts.droptol = 0.005;
 [L, U] = ilu(A, opts);
 nzp = (nnz(L)+nnz(U)-n)/nz0;
 sz4 = sprintf('%4.1f\n',nzp);
 ILU.L = L; ILU.U = U;
 tic
 [sol,res4,its] = fgmres(A, ILU,'precLU', rhs, sol0,im,maxits,tolIts) ;
 toc 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
semilogy(res1,'m--','marker','+','LineWidth',2,'color','k','linewidth',2)
     hold on
     tx = -0.25+length(res1);      ty = res1(end)/5;
     text(tx,ty,sz1,'fontsize',16);
%%second
     semilogy(res2,'g-.','marker','v','LineWidth',2,'color','r','linewidth',2)
     tx = -0.25+length(res2);      ty = res1(end)/5;
     text(tx,ty,sz2,'fontsize',16);
%% third     
semilogy(res3,'b-','marker','d','LineWidth',2,'color','b','linewidth',2)
     tx = -0.25+length(res3);      ty = res1(end)/5;
     text(tx,ty,sz3,'fontsize',16);
%% fourth
semilogy(res4,'r-','marker','o','LineWidth',2,'color','m','linewidth',2)
     tx = -0.25+length(res4);      ty = res1(end)/5;
     text(tx,ty,sz4,'fontsize',16);
%% done - labels and legend 
axis([0 35 5.e-05 1.0e+06])
xlabel('Iterations','fontsize',18) 
ylabel('Residual norm','fontsize',18) 
h1=legend('SSOR(10)','ILU(0)','ILU(1)','ilu-crout')
     hold off 
set(h1,'fontsize',18)
